package ru.myitschool.work.ui.qr.result

import ru.myitschool.work.ui.qr.scan.QrScanDestination
import androidx.navigation.fragment.findNavController
import ru.myitschool.work.SessionManager
import ru.myitschool.work.api.OpenDoorRequest
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.util.Base64
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.lifecycle.lifecycleScope
import kotlinx.coroutines.launch
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import ru.myitschool.work.R
import ru.myitschool.work.api.ApiService
import ru.myitschool.work.core.Constants
import ru.myitschool.work.databinding.FragmentQrScanResultBinding

class QrResult : Fragment(R.layout.fragment_qr_scan_result) {
    private var _binding: FragmentQrScanResultBinding? = null
    private val binding get() = _binding!!

    private lateinit var apiService: ApiService

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentQrScanResultBinding.inflate(inflater, container, false)
        apiService = Retrofit.Builder()
            .baseUrl(Constants.SERVER_ADDRESS)
            .addConverterFactory(GsonConverterFactory.create())
            .build()
            .create(ApiService::class.java)

        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Получаем данные из аргументов
        val qrData = QrScanDestination.getDataIfExist(requireArguments())
        Log.d("QrResult", "QR Data: $qrData") // Логируем полученные данные

        if (qrData != null) {
            binding.result.text = "Результат сканирования: $qrData" // Отображаем результат сканирования
            sendRequestToServer(qrData)
        } else {
            binding.result.text = "Вход был отменён/Operation was cancelled"
        }

        binding.close.setOnClickListener {
            findNavController().popBackStack()
        }
    }

    private fun sendRequestToServer(qrData: String) {
        lifecycleScope.launch {
            try {
                val qrValue = qrData.toLong() // Преобразуем данные QR-кода в Long
                val login = SessionManager.userLogin ?: "default_login" // Замените на ваш логин
                val password = "your_password" // Замените на ваш пароль
                val authHeader = "Basic " + Base64.encodeToString("$login:$password".toByteArray(), Base64.NO_WRAP)

                // Логируем данные перед отправкой
                Log.d("QrResult", "Sending request with QR value: $qrValue and authHeader: $authHeader")

                // Создаем объект запроса
                val request = OpenDoorRequest(qrValue)

                // Вызываем метод openDoor с правильными параметрами
                val response = apiService.openDoor(authHeader, request)

                // Логируем код ответа и тело ответа
                Log.d("QrResult", "Response code: ${response.code()}")
                Log.d("QrResult", "Response body: ${response.body()}")

                if (response.isSuccessful) {
                    binding.result.text = "Door Opened" // Сообщение о том, что дверь открыта
                } else {
                    binding.result.text = "Door Closed" // Сообщение о том, что дверь закрыта
                }
            } catch (e: NumberFormatException) {
                binding.result.text = "Некорректные данные QR-кода"
            } catch (e: Exception) {
                binding.result.text = "Что-то пошло не так/Something went wrong: ${e.message}"
                Log.e("QrResult", "Error sending request to server", e)
            }
        }
    }

    override fun onDestroyView() {
        _binding = null
        super.onDestroyView()
    }
}